use std::path::PathBuf;

use lazy_static::lazy_static;
use serde::Deserialize;

use crate::install::languages::LanguageDefs;

lazy_static! {
    static ref XDG: xdg::BaseDirectories =
        xdg::BaseDirectories::with_prefix("chrysopoeia").unwrap();
}

#[derive(Debug, Default, Deserialize)]
pub struct Settings {
    #[serde(default = "default_data_dir")]
    pub data_dir: PathBuf,

    #[serde(default)]
    pub install: InstallSettings,

    #[serde(rename = "languages")]
    #[serde(default)]
    pub user_defs: LanguageDefs,
}

#[derive(Debug, Default, Deserialize)]
pub struct InstallSettings {
    #[serde(alias = "cc")]
    pub compiler: Option<String>,
}

fn default_data_dir() -> PathBuf {
    if cfg!(debug_assertions) {
        "share".into()
    } else {
        XDG.get_data_home()
    }
}

pub fn default_config() -> PathBuf {
    if cfg!(debug_assertions) {
        "chrysopoeia.toml".into()
    } else {
        XDG.get_config_file("config.toml")
    }
}

impl Settings {
    pub fn parser_dir(&self) -> PathBuf {
        self.data_dir.join("parsers")
    }

    pub fn language_file(&self) -> PathBuf {
        self.data_dir.join("languages.toml")
    }
}
