use std::{
    fs::read_to_string,
    io,
    path::{Path, PathBuf},
};

#[derive(Debug, thiserror::Error)]
#[error("{path:?}: {e}")]
pub struct TomlError {
    path: PathBuf,
    e: TomlErrorInner,
}

#[derive(Debug, thiserror::Error)]
enum TomlErrorInner {
    #[error(transparent)]
    Io(#[from] io::Error),

    #[error(transparent)]
    Deserialize(#[from] toml::de::Error),
}

pub fn load_toml<T>(path: &Path) -> Result<T, TomlError>
where
    T: serde::de::DeserializeOwned,
{
    load_toml_inner(path).map_err(|e| TomlError {
        path: path.to_owned(),
        e,
    })
}

fn load_toml_inner<T>(path: &Path) -> Result<T, TomlErrorInner>
where
    T: serde::de::DeserializeOwned,
{
    Ok(toml::from_str(&read_to_string(path)?)?)
}
